@@ -7,13 +7,12 @@
 
 import json
 import os
-import stat
 import tempfile
 import time
 
 import pytest
 
-from .puppet_logger import PuppetLogger
+from .puppet_logger import PuppetLogger, onerror
 
 
 def test_puppet_logger_01(tmp_path):
@@ -24,19 +23,21 @@ def test_puppet_logger_01(tmp_path):
     assert plog.working_path is not None
     assert os.path.isdir(plog.working_path)
     assert plog._base is not None
-    assert os.listdir(plog._base)
+    assert any(os.scandir(plog._base))
     plog.close()
-    assert os.listdir(plog._base)
+    assert any(os.scandir(plog._base))
     assert plog.closed
     with pytest.raises(AssertionError):
         plog.add_log("test")
     assert plog.log_length("missing") is None
 
+
 def test_puppet_logger_02(tmp_path):
     """test PuppetLogger.add_log() and PuppetLogger.available_logs()"""
     with PuppetLogger(base_path=str(tmp_path)) as plog:
         assert not plog._logs
         assert not plog.available_logs()
+        assert not any(plog.files)
         plog.add_log("test_new")  # non-existing log
         assert "test_new" in plog.available_logs()
         assert os.path.isfile(plog.get_fp("test_new").name)
@@ -44,10 +45,12 @@ def test_puppet_logger_02(tmp_path):
             in_fp.write(b"blah")
             plog.add_log("test_existing", logfp=in_fp)
         assert len(plog.available_logs()) == 2
+        assert len(tuple(plog.files)) == 2
         assert os.path.isfile(plog.get_fp("test_existing").name)
         assert plog.log_length("test_new") == 0
         assert plog.log_length("test_existing") == 4
 
+
 def test_puppet_logger_03(tmp_path):
     """test PuppetLogger.clean_up()"""
     with PuppetLogger(base_path=str(tmp_path)) as plog:
@@ -56,15 +59,16 @@ def test_puppet_logger_03(tmp_path):
         assert plog.working_path is not None
         assert os.path.isdir(plog.working_path)
         assert plog._base is not None
-        assert os.listdir(plog._base)
+        assert any(os.scandir(plog._base))
         plog.add_log("test_new")
         plog.clean_up()
         assert plog.closed
-        assert not os.listdir(plog._base)
+        assert not any(os.scandir(plog._base))
         assert plog.working_path is None
         assert plog.closed
         assert not plog._logs
 
+
 def test_puppet_logger_04(tmp_path):
     """test PuppetLogger.reset()"""
     with PuppetLogger(base_path=str(tmp_path)) as plog:
@@ -77,6 +81,7 @@ def test_puppet_logger_04(tmp_path):
         assert plog._base is not None
         assert len(os.listdir(plog._base)) == 1
 
+
 def test_puppet_logger_05(tmp_path):
     """test PuppetLogger.clone_log()"""
     with PuppetLogger(base_path=str(tmp_path)) as plog:
@@ -92,6 +97,7 @@ def test_puppet_logger_05(tmp_path):
         assert os.path.isfile(cloned)
         with open(cloned, "rb") as log_fp:
             assert log_fp.read() == b"test1"
+        os.remove(cloned)
         # test target exists
         target = tmp_path / "target.txt"
         target.touch()
@@ -111,6 +117,7 @@ def test_puppet_logger_05(tmp_path):
         with open(cloned, "rb") as log_fp:
             assert log_fp.read() == b"1test2test3"
         assert plog.log_length("test_new") == 15
+        os.remove(cloned)
         # test non existent log
         assert plog.clone_log("no_log") is None
         # test empty log
@@ -118,6 +125,8 @@ def test_puppet_logger_05(tmp_path):
         cloned = plog.clone_log("test_empty")
         assert os.path.isfile(cloned)
         assert not os.stat(cloned).st_size
+        os.remove(cloned)
+
 
 def test_puppet_logger_06(tmp_path):
     """test PuppetLogger.save_logs()"""
@@ -126,7 +135,7 @@ def test_puppet_logger_06(tmp_path):
         # save when there are no logs
         dest = tmp_path / "dest"
         plog.save_logs(str(dest))
-        assert not any(dest.glob("*"))
+        assert not any(dest.iterdir())
         plog.reset()
         dest.rmdir()
         # add small log
@@ -156,9 +165,9 @@ def test_puppet_logger_06(tmp_path):
         meta_test.unlink()
         # check saved file count
         assert len(plog.available_logs()) == 5
-        assert len(tuple(dest.glob("*"))) == 6
+        assert len(tuple(dest.iterdir())) == 6
         # verify meta data was copied
-        meta_file = (dest / PuppetLogger.META_FILE)
+        meta_file = dest / PuppetLogger.META_FILE
         assert meta_file.is_file()
         meta_map = json.loads(meta_file.read_text())
         assert len(meta_map) == 5
@@ -168,22 +177,39 @@ def test_puppet_logger_06(tmp_path):
         assert os.stat(plog.get_fp("test_2").name).st_size == 7
         assert os.stat(plog.get_fp("test_3").name).st_size == 500 * 1234
 
+
 def test_puppet_logger_07(mocker, tmp_path):
     """test PuppetLogger.save_logs() rr trace directory"""
     fake_ck = mocker.patch("ffpuppet.puppet_logger.check_output", autospec=True)
     with PuppetLogger(base_path=str(tmp_path)) as plog:
         os.makedirs(os.path.join(plog.working_path, plog.PATH_RR, "latest-trace"))
         plog.close()
-        # call to rr failed
+        # test call to rr failing
         fake_ck.side_effect = OSError
-        plog.save_logs(str(tmp_path / "dest1"))
+        plog.save_logs(str(tmp_path / "dest1"), rr_pack=True)
         assert fake_ck.call_count == 1
         assert not plog._rr_packed
-        # call to rr passed
+        # test call to rr passing
         fake_ck.side_effect = None
-        plog.save_logs(str(tmp_path / "dest2"))
+        plog.save_logs(str(tmp_path / "dest2"), rr_pack=True)
         assert fake_ck.call_count == 2
         assert plog._rr_packed
+        # test 'taskcluster-build-task' copied
+        bin_path = tmp_path / "bin_path"
+        bin_path.mkdir()
+        (bin_path / "taskcluster-build-task").write_text("task-info\n")
+        plog.save_logs(str(tmp_path / "dest3"), bin_path=str(bin_path))
+        assert (
+            tmp_path
+            / "dest3"
+            / "rr-traces"
+            / "latest-trace"
+            / "files.mozilla"
+            / "taskcluster-build-task"
+        ).is_file()
+        assert fake_ck.call_count == 2
+        assert plog._rr_packed
+
 
 def test_puppet_logger_08(tmp_path):
     """test PuppetLogger.add_log() with file not on disk"""
@@ -193,38 +219,34 @@ def test_puppet_logger_08(tmp_path):
             with pytest.raises(IOError, match="log file None does not exist"):
                 plog.get_fp("test")
 
-def test_puppet_logger_09(tmp_path):
-    """test PuppetLogger.clean_up() with inaccessible directory"""
-    with PuppetLogger(base_path=str(tmp_path)) as plog:
-        plog.add_log("test")
-        os.chmod(plog.working_path, stat.S_IRUSR)
-        working_path = plog.working_path
-        plog.close()
-        try:
-            assert os.path.isdir(plog.working_path)
-            with pytest.raises(OSError):
-                plog.clean_up()
-            assert plog.working_path is not None
-            plog.clean_up(ignore_errors=True)
-            assert plog.working_path is None
-            assert os.path.isdir(working_path)
-        finally:
-            os.chmod(working_path, stat.S_IRWXU)
 
-def test_puppet_logger_10(mocker, tmp_path):
-    """test PuppetLogger.clean_up() with in-use file"""
+def test_puppet_logger_09(mocker, tmp_path):
+    """test PuppetLogger.clean_up() with in-use file or inaccessible directory"""
     fake_rmtree = mocker.patch("ffpuppet.puppet_logger.rmtree", autospec=True)
     with PuppetLogger(base_path=str(tmp_path)) as plog:
         plog.add_log("test")
+        working_path = plog.working_path
+        # test with ignore_errors=False
         fake_rmtree.side_effect = OSError
         with pytest.raises(OSError):
-            plog.clean_up(wait_delay=0)
+            plog.clean_up()
+        assert fake_rmtree.call_count == 2
+        fake_rmtree.assert_called_with(
+            working_path, ignore_errors=False, onerror=onerror
+        )
         assert plog.working_path is not None
+        fake_rmtree.reset_mock()
+        # test with ignore_errors=True
         fake_rmtree.side_effect = None
-        plog.clean_up()
-        plog.close()
+        plog.clean_up(ignore_errors=True)
+        assert fake_rmtree.call_count == 1
+        fake_rmtree.assert_called_with(
+            working_path, ignore_errors=True, onerror=onerror
+        )
+        assert plog.working_path is None
 
-def test_puppet_logger_11(tmp_path):
+
+def test_puppet_logger_10(tmp_path):
     """test PuppetLogger.add_path()"""
     with PuppetLogger(base_path=str(tmp_path)) as plog:
         path = plog.add_path("test")
@@ -234,5 +256,5 @@ def test_puppet_logger_11(tmp_path):
         plog.close()
         dest = tmp_path / "dest"
         plog.save_logs(str(dest))
-        assert any(dest.glob("test"))
-        assert any(dest.glob("test/simple.txt"))
+        assert (dest / "test").is_dir()
+        assert (dest / "test" / "simple.txt").is_file()
